/* avagl: avagl.cc
 * 
 * Copyright 2003,2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "parameters.hh"
#include "random.hh"
#include "sandbox.hh"
#include "surface_profile.hh"
#include <sstream>
#include <iomanip>

using namespace gltk;

class AvaGUI
	: public Window
{
private:
	Sandbox sb;
	SurfaceProfile sp;
	bool running;
	bool keep_running;
	unsigned delay;

	void prob_change(Adjustment * a)
	{
		sb.set_prob(a->get_value());
	}

	int step;
	void step_change(SpinButton * s)
	{
		step = s->get_value_as_int();
	}

	bool new_delay;
	void delay_change(SpinButton * s)
	{
		delay = s->get_value_as_int();
		new_delay = true;
	}

	bool fresh;
	void toggle_fresh(CheckButton * t)
	{
		fresh = t->get_active();
	}

	bool single;
	void toggle_single(CheckButton * t)
	{
		single = t->get_active();
	}

	bool spin;
	void toggle_spin(CheckButton * t)
	{
		spin = t->get_active();
	}

	int done;

	sigc::signal<void> sig_update_count;
	void update_count(Label * l)
	{
		std::ostringstream s;
		s << std::setw(8) << sb.get_count() << std::ends;
		l->set_text(s.str());
	}

	bool do_avalanche()
	{
		if (done < step) {
			if (! fresh) {
				sb.iterate();
				sig_update_count.emit();
				if (spin) sp.rotate_y(1);
				sp.redraw();
			}
			done ++;
		} else {
			if (fresh) {
				sb.start();
				sig_update_count.emit();
				if (spin) sp.rotate_y(1);
				sp.redraw();
			}
			done = 0;
			if (single) stop_avalanche();
		}
		if (running) {
			if (new_delay) {
				if (delay == 0)	SignalIdle::connect(sigc::mem_fun(this, & AvaGUI::do_avalanche));
				else SignalTimeout::connect(sigc::mem_fun(this, & AvaGUI::do_avalanche), delay);
				new_delay = false;
				return false;
			}
			return true;
		}
		return false;
	}

	void start_avalanche()
	{
		done = 0;
		if (!running) {
			running= true;
			if (delay == 0)	SignalIdle::connect(sigc::mem_fun(this, & AvaGUI::do_avalanche));
			else SignalTimeout::connect(sigc::mem_fun(this, & AvaGUI::do_avalanche), delay);
			new_delay = false;
		}
	}

	void stop_avalanche()
	{
		done = step;
		running = false;
	}

public:
	AvaGUI(unsigned w, unsigned l) :
		sp(w, l)
	{
		// initialize sandbox
		sb.initialize(w, l);

		sp.set_profile(sb.get_prof());
		sp.set_active(sb.get_active());
		sp.rotate_y(135);

		// EventBox * eb;
		
		Box * vb = new Box(Box::VERTICAL);
		add(vb);

		// Upper Panel
		Box * hb = new Box(Box::HORIZONTAL);
		vb->pack_start(hb);

		// Prob control
		Adjustment * a = new Adjustment(0.5, 0, 1.0, 0.01, 0.05, 0);
		Scale * v = new Scale(a);
		hb->pack_start(v);
		// v->set_digits(2);
		a->signal_value_changed().connect(sigc::bind(sigc::mem_fun(this, & AvaGUI::prob_change), a));

		// SurfaceProfile Widget
		hb->pack_start(& sp);

		// Lower Panel (the Buttons)
		hb = new Box(Box::HORIZONTAL);
		vb->pack_start(hb);

		// Quit Button
		Button * b = new Button("quit");
		hb->pack_start(b);
		b->signal_clicked().connect(sigc::mem_fun(this, & AvaGUI::hide));

		// Avalanche Button
		b = new Button("aval");
		hb->pack_start(b);
		b->signal_clicked().connect(sigc::mem_fun(this, & AvaGUI::start_avalanche));

		// Stop Button
		b = new Button("stop");
		hb->pack_start(b);
		b->signal_clicked().connect(sigc::mem_fun(this, & AvaGUI::stop_avalanche));

		// Refreshing Toggle
		CheckButton * t = new CheckButton("fres");
		hb->pack_start(t);
		t->set_active(false);
		fresh = false;
		t->signal_toggled().connect(sigc::bind(sigc::mem_fun(this, & AvaGUI::toggle_fresh), t));

		// Single Shot Toggle
		t = new CheckButton("sing");
		hb->pack_start(t);
		t->set_active(false);
		single = false;
		t->signal_toggled().connect(sigc::bind(sigc::mem_fun(this, & AvaGUI::toggle_single), t));

		// Spin Toggle
		t = new CheckButton("spin");
		hb->pack_start(t);
		t->set_active(false);
		spin = false;
		t->signal_toggled().connect(sigc::bind(sigc::mem_fun(this, & AvaGUI::toggle_spin), t));

		hb = new Box(Box::HORIZONTAL);
		vb->pack_start(hb);

		// Avalanche Count Lable 
		Label * lb = new Label("         0");
		hb->pack_start(lb);
		sig_update_count.connect(sigc::bind(sigc::mem_fun(this, & AvaGUI::update_count), lb));

		// Delay SpinButton and Adjustment
		SpinButton * s = new SpinButton(100, 0);
		s->configure(100, 0, 3000, 1, 10, 0);
		s->set_width_chars(4);
		hb->pack_start(s);
		s->signal_value_changed().connect(sigc::bind(sigc::mem_fun(this, & AvaGUI::delay_change), s));

		// Step SpinButton and Adjustment
		s = new SpinButton(1, 0);
		s->configure(10, 1, 1000, 1, 10, 0);
		s->set_width_chars(4);
		hb->pack_start(s);
		s->signal_value_changed().connect(sigc::bind(sigc::mem_fun(this, & AvaGUI::step_change), s));

		// Lower Panel finish
		// All finish
		// tp->enable();
		// destroy.connect(Main::quit.slot());
		sb.start();
		done = 0;
		step = 10;
		running = false;
		keep_running = false;
		delay = 100;
		// Main::idle.connect(slot(this, &AvaGUI::do_avalanche));
		sp.show();//show_all();
	}
	~AvaGUI() {}


};

int main(int argc, char *argv[], char *envp[])
{
	Parameters arg(argc, argv, envp);
	Main m(argc, argv);
	AvaGUI ag(32, 64);
	Main::run(ag);
	return 0;
}
